/* iselect_readme.c -- automatically generated by asc2c */

#include <stdio.h>

char *iSelect_README[] = {\
"  <b>    _ ____       _           _   </b>",
"  <b>   (_) ___|  ___| | ___  ___| |_ </b>",
"  <b>  / /\\___ \\ / _ \\ |/ _ \\/ __| __|</b>",
"  <b> / /  ___) |  __/ |  __/ (__| |_ </b>",
"  <b>(_(  |____/ \\___|_|\\___|\\___|\\__|</b>",
"",
"  <b>iSelect -- Interactive Selection Tool</b>",
"",
"  iSelect is an interactive line selection tool for ASCII files, operating",
"  via a full-screen Curses-based terminal session. It can be used either as",
"  an user interface frontend controlled by a Bourne-Shell/Perl/Tcl backend",
"  as its control script or in batch mode as a pipeline filter (usually",
"  between grep and the final executing command).   ",
"  ",
"  Version 1.4.0 (08-Jul-2007)",
"",
"  The latest release can be found on",
"  http://www.ossp.org/pkg/tool/iselect/",
"",
"  Copyright (c) 1997-2007 Ralf S. Engelschall <rse@engelschall.com>",
"",
"  This program is free software; it may be redistributed and/or modified ",
"  only under the terms of the GNU General Public License, which may be found ",
"  in the iSelect source distribution.  Look at the file COPYING for details. ",
"",
"  This program is distributed in the hope that it will be useful, but ",
"  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY",
"  or FITNESS FOR A PARTICULAR PURPOSE.  See the the GNU General Public ",
"  License for more details.",
"",
"                              Ralf S. Engelschall",
"                              rse@engelschall.com",
"                              www.engelschall.com",
"",
NULL };

/*EOF*/
